/**
 * 
 */
package de.fhg.fokus.ngni.odysseus.service.messaging;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

import org.csapi.schema.parlayx.multimedia_messaging.v3_0.DeliveryInformation;
import org.csapi.wsdl.parlayx.multimedia_messaging.send.v3_1.service.PolicyException;
import org.csapi.wsdl.parlayx.multimedia_messaging.send.v3_1.service.SendMessageService;
import org.csapi.wsdl.parlayx.multimedia_messaging.send.v3_1.service.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fhg.fokus.ngni.odysseus.service.messaging.beans.MessageStatus;

/**
 * @author alb
 *
 */
public class Messaging {
	
	/** The Constant Log. */
	private final Logger log = LoggerFactory.getLogger(Messaging.class);
	
	/** The Send Message Service. */
	private SendMessageService service;
	
	public Messaging() throws MalformedURLException{
		this(new URL("http://ims-as1-dv:8080/opense/MultimediaMessaging/SendMessageService?wsdl"));
	}

	public Messaging(URL url){
			service = new SendMessageService(url, new QName("http://www.csapi.org/wsdl/parlayx/multimedia_messaging/send/v3_1/service", "SendMessageService"));
	}
	
	public String sendMessage(String[] recipients, String sender, String content){
		//PARAMS:
		//String[] recipients = {"sip:alb-test@fokus.fraunhofer.de"};
		//String sender = "sip:alb@fokus.fraunhofer.de";
		String subject = "NONE";
		//String content = "TESTMESSAGE";
		
		log.debug("wsdlLocation: " + service.getWSDLDocumentLocation());
		//service does not work - creating directly a SoapConnection:
		
		String wsdlLocation = service.getWSDLDocumentLocation().toString();
		
		String namespace = "http://www.csapi.org/schema/parlayx/multimedia_messaging/send/v3_1/local";
		
			
			try {
				SOAPConnectionFactory soapConnFactory = SOAPConnectionFactory.newInstance();
				SOAPConnection connection = soapConnFactory.createConnection();
				MessageFactory messageFactory = MessageFactory.newInstance();

				SOAPMessage message = messageFactory.createMessage();
				SOAPPart soapPart = message.getSOAPPart();
				SOAPEnvelope envelope = soapPart.getEnvelope();
				SOAPBody body = envelope.getBody();
			
				// Create the main element and namespace
				SOAPElement soapBody = body.addChildElement(envelope.createName("sendMessage", "loc", namespace));
				
				for (String sipUri: recipients) {
					soapBody.addChildElement("addresses", "loc").addTextNode(sipUri);
				}
				soapBody.addChildElement("senderAddress", "loc").addTextNode(sender);
				
				AttachmentPart attachment = message.createAttachmentPart();
				attachment.setContent(content, "text/plain");		 
				message.addAttachmentPart(attachment);		
				
				message.saveChanges();
				log.debug("\nREQUEST:");
				if(log.isDebugEnabled())
					message.writeTo(System.out);
				
				SOAPMessage m = connection.call(message, wsdlLocation);
				
				log.debug("\nRESPONSE:");
				if(log.isDebugEnabled())
					m.writeTo(System.out);
				
				String result = null;
				for (Iterator<Node> i = m.getSOAPBody().getChildElements(); i.hasNext();){
					Node n = i.next();
					if(n.getFirstChild().getNodeName().contains("result"));
						result = n.getFirstChild().getTextContent();
				}
				
				connection.close();
				return result;
			} catch (UnsupportedOperationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SOAPException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		
		return null;
	}
	
	public MessageStatus[] getDeliveryStatus(String messageId){
		
		try {
			List<DeliveryInformation> di = service.getSendMessage().getMessageDeliveryStatus(messageId);
	
			MessageStatus messageStatus[] = new MessageStatus[di.size()];
			
			int i=0;
			for(Iterator<DeliveryInformation> it = di.iterator(); it.hasNext();){
				DeliveryInformation data = it.next();
				String adressUri = (data.getAddress()!=null) ? data.getAddress().toString() : null;
				String mmValue = (data.getDeliveryStatus()!=null) ? data.getDeliveryStatus().value() : null;

				messageStatus[i++] = new MessageStatus(adressUri, mmValue);
			}

			return messageStatus;
		} catch (PolicyException e) {
			log.error("policy exception:");
			log.error("\t-   id: "+ e.getFaultInfo().getMessageId());
			//log.error("\t- text: "+ ExceptionHelper.getMessage(e.getFaultInfo()));
		} catch (ServiceException e) {
			log.error("service exception");
			log.error("\t-   id: "+ e.getFaultInfo().getMessageId());
			//log.error("\t- text: "+ ExceptionHelper.getMessage(e.getFaultInfo()));
		} catch (Throwable e) {
			e.printStackTrace();
		}
		return null;
	}
	
}
